package org.seasar.javelin.jmx;

import java.lang.management.ManagementFactory;

import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.seasar.javelin.jmx.bean.Component;
import org.seasar.javelin.jmx.bean.ComponentMBean;
import org.seasar.javelin.jmx.bean.Invocation;
import org.seasar.javelin.jmx.bean.InvocationMBean;

public class S2JmxJavelinRecorder
{
    /** vbgtH[MBeanT[o */
    private static MBeanServer               server_     = ManagementFactory.getPlatformMBeanServer();

    /**
     * \bhR[c[̋L^pIuWFNgB
     */
    private static ThreadLocal<CallTree>     callTree_   = new ThreadLocal<CallTree>() {
                                                             protected synchronized CallTree initialValue( )
                                                             {
                                                                 return null;
                                                             }
                                                         };

    /**
     * \bȟĂяoIuWFNgB
     */
    private static ThreadLocal<CallTreeNode> callerNode_ = new ThreadLocal<CallTreeNode>() {
                                                             protected synchronized CallTreeNode initialValue( )
                                                             {
                                                                 return null;
                                                             }
                                                         };

    /**
     * OB
     * 
     * @param invocationBean
     */
    public static void preProcess(
    		String className
    		, String methodName
    		, S2JmxJavelinConfig config)
    {
        try
        {
            Component componentBean = MBeanManager.getComponent(className);
            String name = config.getDomain() + ".component:type="
                    + ComponentMBean.class.getName() + ",class=" + className;
            ObjectName componentName = new ObjectName(name);
            if (componentBean == null)
            {
                componentBean = new Component(componentName, className);

                if (server_.isRegistered(componentName))
                {
                    server_.unregisterMBean(componentName);
                }
                server_.registerMBean(componentBean, componentName);
                MBeanManager.setComponent(className, componentBean);
            }

            Invocation invocationBean = componentBean.getInvocation(methodName);
            name = config.getDomain() + ".invocation:type="
                    + InvocationMBean.class.getName() + ",class=" + className
                    + ",method=" + methodName;
            ObjectName objName = new ObjectName(name);

            if (invocationBean == null)
            {
                invocationBean = 
                	new Invocation(
                		objName
                		, componentName
                		, className
                		, methodName
                		, config.getIntervalMax()
                		, config.getThrowableMax()
                		, config.getRecordThreshold()
                		, config.getAlarmThreshold());

                componentBean.addInvocation(invocationBean);
                if (server_.isRegistered(objName))
                {
                    server_.unregisterMBean(objName);
                }
                server_.registerMBean(invocationBean, objName);
            }

            // Ăяo擾B
            CallTreeNode node = callerNode_.get();

            if (node == null)
            {
                // Ăяo̓R[c[B
                CallTree tree = new CallTree();
                callTree_.set(tree);

                node = new CallTreeNode();
                node.setStartTime(System.currentTimeMillis());
                tree.setRootNode(node);
            }
            else
            {
                CallTreeNode parent = node;
                node = new CallTreeNode();
                node.setStartTime(System.currentTimeMillis());
                parent.addChild(node);
                
                invocationBean.addCaller(parent.getInvocation());
            }

            node.setInvocation(invocationBean);

            // ĂяoA
            // 񃍃Oǒ͎ĂяoƂĎgp邽߂ɕۑB
            callerNode_.set(node);
        }
        catch (Exception ex)
        {
            // zO̗Oꍇ͕WG[o͂ɏo͂ĂB
            ex.printStackTrace();
        }
    }

    /**
     * ㏈i{jB
     * @param spent
     */
    public static void postProcess(S2JmxJavelinConfig config)
    {
        try
        {
            // Ăяo擾B
            CallTreeNode node = callerNode_.get();
            if (node == null)
            {
                // Ăяo񂪎擾łȂꍇ͏LZB
                // (ʃCŗOꍇ̂߁B)
                return;
            }

            node.setEndTime(System.currentTimeMillis());

            CallTreeNode parent = node.getParent();
            if (parent != null)
            {
                callerNode_.set(parent);
            }
            else
            {
                // [gm[hł̌oߎԂ臒l𒴂Ăꍇ́A
                // gUNVL^B
            	if (node.getAccumulatedTime() >= node.getInvocation().getRecordThreshold())
                {
                    recordTransaction(node);
                }
            	
            	if (node.getAccumulatedTime() >= node.getInvocation().getAlarmThreshold())
            	{
                    S2JmxJavelinFileGenerator generator = 
                    	new S2JmxJavelinFileGenerator(config.getJavelinFileDir());
                    generator.generateJaveinFile(callTree_.get(), node);

                    sendExceedThresholdAlarm(node);
            	}
            	
                callerNode_.set(null);
            }
        }
        catch (Exception ex)
        {
            // zO̗Oꍇ͕WG[o͂ɏo͂ĂB
            ex.printStackTrace();
        }
    }

	/**
     * ㏈i{sjB
     * @param cause
     */
    public static void postProcess(Throwable cause)
    {
        try
        {
            // Ăяo擾B
            CallTreeNode node = callerNode_.get();
            if (node == null)
            {
                // Ăяo񂪎擾łȂꍇ͏LZB
                // (łɋL^ς݂̗Ô߁B)
                return;
            }

            // OL^ĂB
            node.getInvocation().addThrowable(cause);

            //ĂяoĂB
            callerNode_.set(null);
        }
        catch (Exception ex)
        {
            // zO̗Oꍇ͕WG[o͂ɏo͂ĂB
            ex.printStackTrace();
        }
    }

    /**
     * gUNVL^B
     * 
     * @param node
     */
    private static void recordTransaction(CallTreeNode node)
    {
        Invocation invocation = node.getInvocation();
        invocation.addInterval(node.getElapsedTime());
        if (node.getParent() != null)
        {
            invocation.addCaller(node.getParent().getInvocation());
        }

        for (CallTreeNode child : node.getChildren())
        {
            recordTransaction(child);
        }
    }
    
    private static void sendExceedThresholdAlarm(CallTreeNode node)
	{
        node.getInvocation().sendExceedThresholdAlarm();
        
        for (CallTreeNode child : node.getChildren())
        {
        	sendExceedThresholdAlarm(child);
        }
	}
}
